require_relative './quokkadb/adb_runner.rb'
require_relative './quokkadb/test.rb'
require_relative './quokkadb/test_result.rb'
require_relative './quokkadb/adb_test_parser.rb'
require 'colorize'

class QuokkADB

  attr_accessor :ADBRunner
  attr_accessor :parser

  def initialize(runner:, package:, annotation: nil, shards: nil, index: nil, serial: nil, background_black: false, debug_mode: false)
    @runner = runner
    @package = package
    @annotation = annotation
    @shards = shards
    @index = index
    @serial = serial
    @background_black = background_black
    @debug_mode = debug_mode

    @ADBRunner = ADBRunner.new()
    @parser = ADBTestParser.new()
  end

  def run()
    full_log = ""
    @ADBRunner.callback = lambda { |line|
      full_log += line
      @parser.parse_line(line: line, background_black: @background_black)
    }
    @ADBRunner.run_tests(
      runner: @runner,
      package: @package,
      annotation: @annotation,
      shards: @shards,
      index: @index,
      serial: @serial
    )

    parse_results(full_log)
  end

  def parse_results(full_log)
    full_log.each_line do |log|
      if @debug_mode
        puts(log)
      end

      if (log.include?("FAILURES!!!") or log.include?("Process crashed while executing") or log.include?('Process crashed.') or log.include?("INSTRUMENTATION_FAILED"))
        puts("Log has failures")
        raise 'Log has failures'
      end
    end

    @parser.test_results.each do |result|
      if @debug_mode
        puts result
      end

      if (!result.passed)
        puts("Failed test")
        raise 'Test did not pass'
      end
    end
    puts("All tests successful")
  end

end
